// ignore_for_file: avoid_print, unnecessary_string_interpolations, unnecessary_brace_in_string_interps

import 'package:flutter/services.dart';

class TLTextInputFormatter {
  /// 长度格式化
  static List<TextInputFormatter> maxLengthInputFormatters({
    int? maxLength,
    MaxLengthEnforcement? maxLengthEnforcement,
  }) {
    return [
      LengthLimitingTextInputFormatter(maxLength, maxLengthEnforcement: maxLengthEnforcement),
    ];
  }

  /// 数字格式化
  static List<TextInputFormatter> numberInputFormatters({
    int? maxLength,
    MaxLengthEnforcement? maxLengthEnforcement,
    int? maxIntegerLength,
    int? maxDecimalLength,
    bool isAllowDecimal = true,
  }) {
    return [
      LengthLimitingTextInputFormatter(maxLength, maxLengthEnforcement: maxLengthEnforcement),
      isAllowDecimal ? FilteringTextInputFormatter.allow(RegExp("[0-9.]")) : FilteringTextInputFormatter.allow(RegExp("[0-9]")),
      TLNumberTextInputFormatter(
        maxIntegerLength: maxIntegerLength,
        maxDecimalLength: maxDecimalLength,
        isAllowDecimal: isAllowDecimal,
      ),
    ];
  }

  /// 范围数字格式化
  static List<TextInputFormatter> numericalRangeInputFormatters({
    double min = -999999999999999.999999,
    double max = 999999999999999.999999,
    int decimalCount = 6,
  }) {
    return [
      decimalCount > 0 ? FilteringTextInputFormatter.allow(RegExp("[0-9.-]")) : FilteringTextInputFormatter.allow(RegExp("[0-9]")),
      TLNumericalRangeFormatter(min: min, max: max, decimalCount: decimalCount)
    ];
  }
}

/// 限制输入为合法小数
class TLNumberTextInputFormatter extends TextInputFormatter {
  /// [maxIntegerLength]限定整数的最大位数，为null时不限
  /// [maxDecimalLength]限定小数点的最大位数，为null时不限
  /// [isAllowDecimal]是否可以为小数，默认是可以为小数，也就是可以输入小数点
  final int? _maxIntegerLength;
  final int? _maxDecimalLength;
  final bool _isAllowDecimal;

  TLNumberTextInputFormatter({int? maxIntegerLength, int? maxDecimalLength, bool isAllowDecimal = true})
      : _maxIntegerLength = maxIntegerLength,
        _maxDecimalLength = maxDecimalLength,
        _isAllowDecimal = isAllowDecimal;

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    String? value = newValue.text.trim();

    ///去掉前后空格
    int selectionIndex = newValue.selection.end;
    if (_isAllowDecimal) {
      if (value == '.') {
        value = '0.';
        selectionIndex++;
      } else if (value != '' && _isToDoubleError(value)) {
        ///不是double输入数据
        return _oldTextEditingValue(oldValue);
      }

      ///包含小数点
      if (value.contains('.')) {
        int pointIndex = value.indexOf('.');
        String beforePoint = value.substring(0, pointIndex);
        print('$beforePoint');
        String afterPoint = value.substring(pointIndex + 1, value.length);
        print('$afterPoint');

        ///小数点前面没内容补0
        if (beforePoint.isEmpty) {
          value = '0.${afterPoint}';
          selectionIndex++;
        } else {
          ///限定整数位数
          if (null != _maxIntegerLength) {
            if (beforePoint.length > _maxIntegerLength!) {
              return _oldTextEditingValue(oldValue);
            }
          }
        }

        ///限定小数点位数
        if (null != _maxDecimalLength) {
          if (afterPoint.length > _maxDecimalLength!) {
            return _oldTextEditingValue(oldValue);
          }
        }
      } else {
        ///限定整数位数
        if (null != _maxIntegerLength) {
          if (value.length > _maxIntegerLength!) {
            return _oldTextEditingValue(oldValue);
          }
        }
      }
    } else {
      if (value.contains('.') || (value != '' && _isToDoubleError(value)) || (null != _maxIntegerLength && value.length > _maxIntegerLength!)) {
        return _oldTextEditingValue(oldValue);
      }
    }

    return TextEditingValue(
      text: value,
      selection: TextSelection.collapsed(offset: selectionIndex),
    );
  }

  ///返回旧的输入内容
  TextEditingValue _oldTextEditingValue(TextEditingValue oldValue) {
    return TextEditingValue(
      text: oldValue.text,
      selection: TextSelection.collapsed(offset: oldValue.selection.end),
    );
  }

  ///输入内容不能解析成double
  bool _isToDoubleError(String value) {
    try {
      double.parse(value);
    } catch (e) {
      return true;
    }
    return false;
  }
}

class TLNumericalRangeFormatter extends TextInputFormatter {
  final double min;
  final double max;
  final int decimalCount;

  TLNumericalRangeFormatter({required this.min, required this.max, required this.decimalCount});

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    num? value = num.tryParse(newValue.text);
    if (value == null) {
      return newValue;
    } else {
      if (value < min) {
        String tt = min.toStringAsFixed(decimalCount);
        return TextEditingValue(
          text: tt,
          selection: TextSelection.collapsed(offset: tt.length),
        );
      } else {
        if (value > max) {
          return oldValue;
        } else {
          String sValue = newValue.text.trim();
          int pointIndex = sValue.indexOf('.');
          if (pointIndex > -1) {
            String afterPoint = sValue.substring(pointIndex + 1, sValue.length);
            if (afterPoint.length > decimalCount) {
              return oldValue;
            }
          }
          return newValue;
        }
      }
    }
  }
}
